@@ -3,11 +3,11 @@ package Class::Usul::Log;
 use namespace::autoclean;
 
 use Class::Null;
-use Class::Usul::Constants qw( FALSE LOG_LEVELS NUL TRUE );
+use Class::Usul::Constants qw( FALSE LOG_LEVELS NUL SPC TRUE );
 use Class::Usul::Functions qw( merge_attributes untaint_identifier );
 use Class::Usul::Types     qw( Bool EncodingType HashRef
                                LoadableClass LogType Undef );
-use Encode;
+use Encode                 qw( encode );
 use File::Basename         qw( dirname );
 use File::DataClass::Types qw( Path );
 use Scalar::Util           qw( blessed );
@@ -46,10 +46,10 @@ around 'BUILDARGS' => sub {
 
    my $builder = delete $attr->{builder} or return $attr;
    my $config  = $builder->can( 'config' ) ? $builder->config : {};
+   my $cfgattr = [ qw( log_attributes log_class logfile ) ];
 
    merge_attributes $attr, $builder, {}, [ 'debug', 'encoding' ];
-   merge_attributes $attr, $config,  {},
-      [ qw( encoding log_attributes log_class logfile ) ];
+   merge_attributes $attr, $config,  {}, $cfgattr;
 
    return $attr;
 };
@@ -60,28 +60,23 @@ my $_add_methods = sub {
    for my $method (@methods) {
       $class->can( $method )
          or install_sub { into => $class, as => $method, code => sub {
-            my ($self, $text) = @_; $text or return FALSE;
+            my ($self, $text, $opts) = @_; $text or return FALSE;
+
+            $text = ucfirst "${text}"; chomp $text; $text .= "\n";
+
+            if (defined $opts) {
+               my $lead = ucfirst $opts->{leader} // NUL;
+               my $user = $opts->{user} ? '['.$opts->{user}->username.'] '
+                        :         $lead ? SPC
+                                        : NUL;
+
+               $text = "${lead}${user}${text}";
+            }
 
-            $text = "${text}"; chomp $text; $text .= "\n";
             $self->_encoding and $text = encode( $self->_encoding, $text );
             $self->_log->$method( $text );
             return TRUE;
          } };
-
-      my $meth_msg = "${method}_message";
-
-      $class->can( $meth_msg )
-         or install_sub { into => $class, as => $meth_msg, code => sub {
-            my ($self, $opts, $msg) = @_; my $text;
-
-            my $user = $opts->{user} ? $opts->{user}->username : 'unknown';
-
-            $msg ||= NUL; $msg = "${msg}"; chomp $msg;
-            $text  = (ucfirst $opts->{leader} || NUL)."[${user}] ".
-                     (ucfirst $msg || 'no message');
-            $self->$method( $text );
-            return TRUE;
-         } };
    }
 
    return;
@@ -91,7 +86,9 @@ $_add_methods->( __PACKAGE__, LOG_LEVELS );
 
 # Public methods
 sub filehandle {
-   return $_[ 0 ]->_log->output( 'file-out' )->{fh};
+   my $self = shift; $self->_log_class eq 'Log::Handler' or return;
+
+   return $self->_log->output( 'file-out' )->{fh};
 }
 
 sub get_log_attributes {
@@ -99,10 +96,10 @@ sub get_log_attributes {
 
    $self->_log_class eq 'Log::Handler' or return $attr;
 
-   my $fattr   = $attr->{file} ||= {};
-   my $logfile = $fattr->{filename} || $self->_logfile;
+   my $fattr   = $attr->{file} //= {};
+   my $logfile = $fattr->{filename} // $self->_logfile;
 
-   ($logfile and -d dirname( "${logfile}" )) or return;
+   ($logfile and -d dirname( "${logfile}" )) or return $attr;
 
    $fattr->{alias   } = 'file-out';
    $fattr->{filename} = "${logfile}";
@@ -124,23 +121,20 @@ Class::Usul::Log - Create methods for each logging level that encode their outpu
 
 =head1 Synopsis
 
-   use Moo;
    use Class::Usul::Log;
+   use File::DataClass::IO;
 
-   has '_log' => is => 'ro', isa => LogType,
-      lazy    => TRUE,   builder => '_build__log',
-      reader  => 'log', init_arg => 'log';
+   my $file = io [ 't', 'test.log' ];
+   my $log  = Class::Usul::Log->new( encoding => 'UTF-8', logfile => $file );
+   my $text = 'Your error message goes here';
 
-   sub _build__log {
-      my $self = shift; return Class::Usul::Log->new( builder => $self );
-   }
-
-   # Can now call the following
-   $self->log->debug( $text );
-   $self->log->info(  $text );
-   $self->log->warn(  $text );
-   $self->log->error( $text );
-   $self->log->fatal( $text );
+   # Can now call the following. The text will be encoded UTF-8
+   $log->debug( $text ); # Does not log as debug was not true in the constructor
+   $log->info ( $text );
+   $log->warn ( $text );
+   $log->error( $text );
+   $log->alert( $text );
+   $log->fatal( $text );
 
 =head1 Description
 
@@ -156,7 +150,8 @@ Defines the following attributes
 
 =item C<debug>
 
-Debug flag defaults to FALSE
+Debug flag defaults to false. If set to true calls to log at the debug level
+will succeed rather than being ignored
 
 =item C<encoding>
 
@@ -171,6 +166,10 @@ is not provided
 
 Attributes used to create the log object
 
+=item C<log_class>
+
+The classname of the log object. This is loaded on demand
+
 =item C<logfile>
 
 Path to the logfile
@@ -183,22 +182,15 @@ Path to the logfile
 
 Monkey with the constructors signature
 
-=head2 C<BUILD>
-
-Creates a set of methods defined by the C<LOG_LEVELS> constant. The
-method expects C<< $self->log >> and C<< $self->encoding >> to be set.
-It encodes the output string prior calling the log method at the given
-level
-
 =head2 C<filehandle>
 
-Return the loggers file handle
+Return the loggers file handle. This was added for L<IO::Async>, so that we
+can tell it not to close the log file handle when it forks a child process
 
 =head2 get_log_attributes
 
-Returns the hash ref passed to the constructor of the log class. Returns
-undef to indicate no logging, an instance of L<Class::Null> is used
-instead
+Returns the hash reference passed to the constructor of the log class. Returns
+C<undef> to indicate no logging, an instance of L<Class::Null> is used instead
 
 =head1 Diagnostics
 
@@ -210,15 +202,11 @@ None
 
 =item L<Class::Null>
 
-=item L<Class::Usul::Constants>
-
-=item L<Class::Usul::Functions>
-
 =item L<Moo>
 
 =item L<Encode>
 
-=item L<File::DataClass::Types>
+=item L<File::DataClass>
 
 =item L<Log::Handler>
 
